import sqlalchemy as sql
import matplotlib.pyplot as plt
import numpy as np
import time

engine_pao_run = sql.create_engine('mysql+mysqldb://student:2021upol@158.194.88.42:16336/FAST_PAO_RUN')
conn_pao_run = engine_pao_run.connect()
metadata_pao_run = sql.MetaData()
uvledtab = sql.Table('UVLEDTab', metadata_pao_run, autoload=True, autoload_with=engine_pao_run)

engine_pao_calib = sql.create_engine('mysql+mysqldb://student:2021upol@158.194.88.42:16336/FAST_PAO_CALIB')
conn_pao_calib = engine_pao_calib.connect()
metadata_pao_calib = sql.MetaData()
abscalibtab = sql.Table('AbsCalibTab', metadata_pao_calib, autoload=True, autoload_with=engine_pao_calib)
uvledcalibtab = sql.Table('UVLEDCalibTab', metadata_pao_calib, autoload=True, autoload_with=engine_pao_calib)


def get_calib_const(calib_type, chanid, ts):  # Defining a function to obtain the calibration constants
    data = {}
    if calib_type == "abs":
        query = sql.select([abscalibtab]).where(abscalibtab.columns.chanid == chanid)
    elif calib_type == "uvled":
        query = sql.select([uvledcalibtab]).where(uvledcalibtab.columns.chanid == chanid)
    else: return 0
    ResultProxy = conn_pao_calib.execute(query)
    res = ResultProxy.fetchall()
    for r in res: data[r[0]] = r[2]
    return data[ts] if ts in data else data[min(data.keys(), key=lambda k: abs(k - ts))]


timestamp_calib_const = 1595335842
calib_const_id0 = get_calib_const("abs", 0, timestamp_calib_const)
calib_const_id1 = get_calib_const("abs", 1, timestamp_calib_const)
calib_const_id2 = get_calib_const("abs", 2, timestamp_calib_const)
calib_const_id3 = get_calib_const("abs", 3, timestamp_calib_const)  # Defining the calibration constants for each PMT

ts = 1595153614  # Timestamp of the selected UV LED calibration run
signal = sql.select([uvledtab]).where(uvledtab.columns.timestamp >= ts).where(uvledtab.columns.timestamp <= ts + 50)
ResultProxy_signal = conn_pao_run.execute(signal)
rows_signal = ResultProxy_signal.fetchall()  # Importing data from the FAST database

y0 = []
y1 = []
y2 = []
y3 = []
time_list = []
while len(time_list) < 5:
    for row in rows_signal:
        if row[1] == 0:
            y0.append(row[2] / calib_const_id0)
            time_list.append(row[0])
        elif row[1] == 1:
            y1.append(row[2] / calib_const_id1)
        elif row[1] == 2:
            y2.append(row[2] / calib_const_id2)
        elif row[1] == 3:
            y3.append(row[2] / calib_const_id3)  # Creating data lists for all PMT signal values

y0_relative = np.divide(y0, y1)
y1_relative = np.divide(y1, y1)
y2_relative = np.divide(y2, y1)
y3_relative = np.divide(y3, y1)  # Taking their relative values in reference to PMT 1

date = []
for timestamp in time_list:  # Converting timestamp into a suitable format
    date.append(time.strftime('%d-%m-%Y, %H:%M:%S', time.localtime(timestamp)))

values = [{'name': 'PMT 0', 'x': time_list,
           'y': y0_relative, 'color': 'r', 'marker': '^'},
          {'name': 'PMT 1', 'x': time_list,
           'y': y1_relative, 'color': 'b', 'marker': '^'},
          {'name': 'PMT 2', 'x': time_list,
           'y': y2_relative, 'color': 'g', 'marker': '^'},
          {'name': 'PMT 3', 'x': time_list,
           'y': y3_relative, 'color': 'cyan', 'marker': '^'}]

for signal in values:
    plt.scatter(signal['x'], signal['y'],
                color=signal['color'],
                label=signal['name'],
                marker=signal['marker'])
plt.legend()
plt.xlabel("Date")
plt.xticks([time_list[0], time_list[4]],
           [date[0], date[4]], rotation=20)
plt.axhline(np.average(y0_relative), color="black")
plt.axhline(np.average(y1_relative), color="black")
plt.axhline(np.average(y2_relative), color="black")
plt.axhline(np.average(y3_relative), color="black")
plt.ylabel("Relative comparison of all PMTs in reference to PMT 1")
plt.title("An example of a UV LED calibration run")
plt.show()  # Creating the specific plot of the UV LED calibration run
