import sqlalchemy as sql
import matplotlib.pyplot as plt
import numpy as np

engine_pao_run = sql.create_engine('mysql+mysqldb://student:2021upol@158.194.88.42:16336/FAST_PAO_RUN')
conn_pao_run = engine_pao_run.connect()
metadata_pao_run = sql.MetaData()
uvledtab = sql.Table('UVLEDTab', metadata_pao_run, autoload=True, autoload_with=engine_pao_run)

engine_pao_calib = sql.create_engine('mysql+mysqldb://student:2021upol@158.194.88.42:16336/FAST_PAO_CALIB')
conn_pao_calib = engine_pao_calib.connect()
metadata_pao_calib = sql.MetaData()
abscalibtab = sql.Table('AbsCalibTab', metadata_pao_calib, autoload=True, autoload_with=engine_pao_calib)
uvledcalibtab = sql.Table('UVLEDCalibTab', metadata_pao_calib, autoload=True, autoload_with=engine_pao_calib)

engine_fast = sql.create_engine('mysql+mysqldb://student:2021upol@158.194.88.42:16336/FAST')
conn_fast = engine_fast.connect()
metadata_fast = sql.MetaData()
TelescopeConditions = sql.Table('TelescopeConditions', metadata_fast, autoload=True, autoload_with=engine_fast)


def get_calib_const(calib_type, chanid, ts):
    data = {}

    if calib_type == "abs":
        query = sql.select([abscalibtab]).where(abscalibtab.columns.chanid == chanid)
    elif calib_type == "uvled":
        query = sql.select([uvledcalibtab]).where(uvledcalibtab.columns.chanid == chanid)
    else:
        return 0

    ResultProxy = conn_pao_calib.execute(query)
    res = ResultProxy.fetchall()
    for r in res: data[r[0]] = r[2]

    return data[ts] if ts in data else data[min(data.keys(), key=lambda k: abs(k - ts))]


timestamp_calib_const = 1595335842
calib_const_id0 = get_calib_const("abs", 0, timestamp_calib_const)
calib_const_id1 = get_calib_const("abs", 1, timestamp_calib_const)
calib_const_id2 = get_calib_const("abs", 2, timestamp_calib_const)
calib_const_id3 = get_calib_const("abs", 3, timestamp_calib_const)

temp_start_original = 1584944775
temp_end_original = 1594227503
temp_start = 1584944775
temp_end = 1625824752
telecond = sql.select([TelescopeConditions]).where(TelescopeConditions.columns.timestamp >= temp_start).where(
    TelescopeConditions.columns.id <= temp_end)
ResultProxy_telecond = conn_fast.execute(telecond)
rows_telecond = ResultProxy_telecond.fetchall()

timestamp_list = []
tempcam_list = []
for row in rows_telecond:
    timestamp_list.append(row[1])
    tempcam_list.append(row[8])

signal = sql.select([uvledtab]).where(uvledtab.columns.timestamp >= temp_start).where(
    uvledtab.columns.timestamp <= temp_end)
ResultProxy_signal = conn_pao_run.execute(signal)
rows_signal = ResultProxy_signal.fetchall()

y0 = []
y1 = []
y2 = []
y3 = []
time_list = []
for row in rows_signal:
    if row[1] == 0:
        y0.append(row[2] / calib_const_id0)
        time_list.append(row[0])
    elif row[1] == 1:
        y1.append(row[2] / calib_const_id1)
    elif row[1] == 2:
        y2.append(row[2] / calib_const_id2)
    elif row[1] == 3:
        y3.append(row[2] / calib_const_id3)

time_list_refined = time_list[::5]
y0_refined = y0[::5]
y1_refined = y1[::5]
y2_refined = y2[::5]
y3_refined = y3[::5]

index_of0 = y1_refined.index(0)
time_list_refined.pop(index_of0)
y0_refined.pop(index_of0)
y1_refined.pop(index_of0)
y2_refined.pop(index_of0)
y3_refined.pop(index_of0)

temp_list = []
for ts in time_list_refined:
    closest_timestamp = min(timestamp_list, key=lambda x: abs(x - ts))
    index = timestamp_list.index(closest_timestamp)
    tempcam = tempcam_list[index]
    temp_list.append(tempcam)

# temp_list je list teplot odpovídající hodnotám signálu naměřených hodnot y1-3

y0_relative = np.divide(y0_refined, y1_refined)
y1_relative = np.divide(y1_refined, y1_refined)
y2_relative = np.divide(y2_refined, y1_refined)
y3_relative = np.divide(y3_refined, y1_refined)

values = [{'name': 'PMT 0', 'x': temp_list,
           'y': y0_relative, 'color': 'r', 'marker': '^'},
          {'name': 'PMT 1', 'x': temp_list,
           'y': y1_relative, 'color': 'b', 'marker': '^'},
          {'name': 'PMT 2', 'x': temp_list,
           'y': y2_relative, 'color': 'g', 'marker': '^'},
          {'name': 'PMT 3', 'x': temp_list,
           'y': y3_relative, 'color': 'cyan', 'marker': '^'}]

n_bins = 300
for signal in values:
    plt.hist(signal['y'], n_bins, histtype='step', stacked=True, fill=False, range=[0, 3], alpha=0.65,
             color=signal['color'],
             label=signal['name'])
# plt.hist(signal['y'], n_bins, range=[0, 4], facecolor='green', alpha=0.65)
plt.legend()
plt.grid(axis='y')
plt.xlabel("Relative values")
plt.ylabel("Counts")
plt.title("Histogram")
plt.show()
print(y1_refined)
