import sqlalchemy as sql
import matplotlib.pyplot as plt
import numpy as np
import time

engine_pao_run = sql.create_engine('mysql+mysqldb://student:2021upol@158.194.88.42:16336/FAST_PAO_RUN')
conn_pao_run = engine_pao_run.connect()
metadata_pao_run = sql.MetaData()
uvledtab = sql.Table('UVLEDTab', metadata_pao_run, autoload=True, autoload_with=engine_pao_run)

engine_pao_calib = sql.create_engine('mysql+mysqldb://student:2021upol@158.194.88.42:16336/FAST_PAO_CALIB')
conn_pao_calib = engine_pao_calib.connect()
metadata_pao_calib = sql.MetaData()
abscalibtab = sql.Table('AbsCalibTab', metadata_pao_calib, autoload=True, autoload_with=engine_pao_calib)
uvledcalibtab = sql.Table('UVLEDCalibTab', metadata_pao_calib, autoload=True, autoload_with=engine_pao_calib)


def get_calib_const(calib_type, chanid, ts):
    data = {}

    if calib_type == "abs":
        query = sql.select([abscalibtab]).where(abscalibtab.columns.chanid == chanid)
    elif calib_type == "uvled":
        query = sql.select([uvledcalibtab]).where(uvledcalibtab.columns.chanid == chanid)
    else:
        return 0

    ResultProxy = conn_pao_calib.execute(query)
    res = ResultProxy.fetchall()
    for r in res: data[r[0]] = r[2]

    return data[ts] if ts in data else data[min(data.keys(), key=lambda k: abs(k - ts))]


timestamp_calib_const = 1595335842
calib_const_id0 = get_calib_const("abs", 0, timestamp_calib_const)
calib_const_id1 = get_calib_const("abs", 1, timestamp_calib_const)
calib_const_id2 = get_calib_const("abs", 2, timestamp_calib_const)
calib_const_id3 = get_calib_const("abs", 3, timestamp_calib_const)

signal = sql.select([uvledtab])
ResultProxy_signal = conn_pao_run.execute(signal)
rows_signal = ResultProxy_signal.fetchall()

y0 = []
y1 = []
y2 = []
y3 = []
time_list = []
for row in rows_signal:
    if row[1] == 0:
        y0.append(row[2] / calib_const_id0)
        time_list.append(row[0])
    elif row[1] == 1:
        y1.append(row[2] / calib_const_id1)
    elif row[1] == 2:
        y2.append(row[2] / calib_const_id2)
    elif row[1] == 3:
        y3.append(row[2] / calib_const_id3)

time_list_refined = time_list[::5]
y0_refined = y0[::5]
y1_refined = y1[::5]
y2_refined = y2[::5]
y3_refined = y3[::5]

indices = []
for i in y0_refined:
    if i < 70:
        indices.append(y0_refined.index(i))
for index in sorted(indices, reverse=True):
    del y0_refined[index]
    del y1_refined[index]
    del y2_refined[index]
    del y3_refined[index]
    del time_list_refined[index]

del y0_refined[:4]
del y1_refined[:4]
del y2_refined[:4]
del y3_refined[:4]
del time_list_refined[:4]

y0_relative = np.divide(y0_refined, y1_refined)
y1_relative = np.divide(y1_refined, y1_refined)
y2_relative = np.divide(y2_refined, y1_refined)
y3_relative = np.divide(y3_refined, y1_refined)

date = []
for timestamp in time_list_refined:
    date.append(time.strftime('%d-%m-%Y', time.localtime(timestamp)))

values = [{'name': 'PMT 0', 'x': time_list_refined,
           'y': y0_relative, 'color': 'r', 'marker': '^'},
          {'name': 'PMT 1', 'x': time_list_refined,
           'y': y1_relative, 'color': 'b', 'marker': '^'},
          {'name': 'PMT 2', 'x': time_list_refined,
           'y': y2_relative, 'color': 'g', 'marker': '^'},
          {'name': 'PMT 3', 'x': time_list_refined,
           'y': y3_relative, 'color': 'cyan', 'marker': '^'}]

for signal in values:
    plt.scatter(signal['x'], signal['y'],
                color=signal['color'],
                label=signal['name'],
                marker=signal['marker'])
plt.legend()
plt.grid(axis='y')
plt.xlabel("Date")
plt.xticks([time_list_refined[0], time_list_refined[250], time_list_refined[len(time_list_refined) - 5]],
           [date[0], date[250], date[len(time_list_refined) - 5]], rotation=20)
plt.ylabel("Relative comparison of all PMTs in reference to PMT 1")
plt.title("Relative Time Calibration (filtered)")
plt.show()
